@@ -10,7 +10,10 @@ try:
     import importlib.metadata as importlib_metadata
 except ImportError:
     import importlib_metadata as importlib_metadata
-__version__ = importlib_metadata.version(__package__ or __name__)
+try:
+    __version__ = importlib_metadata.version(__package__ or __name__)
+except importlib_metadata.PackageNotFoundError:
+    __version__ = None
 
 # .env load must be before imports that use environment variables
 load_dotenv(find_dotenv(".env"))
@@ -23,5 +26,6 @@ from toolchest_client.api.download import download
 from toolchest_client.api.exceptions import ToolchestException, DataLimitError, ToolchestJobError, \
     ToolchestDownloadError
 from toolchest_client.api.query import Query
-from .tools.api import bowtie2, cellranger_count, kraken2, megahit, shi7, shogun_align, shogun_filter, STAR, test, \
-    unicycler
+from toolchest_client.api.status import Status, get_status
+from .tools.api import alphafold, bowtie2, cellranger_count, kraken2, megahit, shi7, shogun_align, shogun_filter, STAR,\
+    test, unicycler
